package trains.ui;

import java.time.DayOfWeek;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Collection;
import java.util.List;
import java.util.SortedMap;
import java.util.SortedSet;
import java.util.TreeMap;
import java.util.TreeSet;

import trains.model.*;

public class MyController implements Controller {

	private Collection<Train> trains;
	private SortedMap<String, Station> stationMap;

	public MyController(Collection<Station> stations, Collection<Train> trains) {	
		if (stations == null)
			throw new IllegalArgumentException("stations");
		if (trains == null)
			throw new IllegalArgumentException("trains");
		
		this.stationMap = new TreeMap<String, Station>();
		for (Station station : stations) {
			this.stationMap.put(station.toString(), station);
		}
		this.trains = trains;
	}

	public List<Train> trainServing(String stationName1, String stationName2) {
		List<Train> list = new ArrayList<>();
		Station station1 = stationMap.get(stationName1);
		Station station2 = stationMap.get(stationName2);
		for (Train t : trains) {
			List<Station> stazioniServite = t.getServedStations();
			if (stazioniServite.contains(station1) && stazioniServite.contains(station2))
				list.add(t);
		}
		return list;
	}

	public List<Train> trainServing(String station1, String station2,
			LocalDate day) {
		List<Train> list = new ArrayList<>();
		List<Train> allTrains = trainServing(station1, station2);
		DayOfWeek d = day.getDayOfWeek();
		for (Train t : allTrains) {
			if (containsDay(t.getDays(), d))
				list.add(t);
		}
		return list;
	}

	@Override
	public SortedSet<String> getSortedStations() {
		return new TreeSet<String>(stationMap.keySet());
	}
	
	private static boolean containsDay(DayOfWeek[] days, DayOfWeek day) {
		for (DayOfWeek current : days) {
			if (current == day) {
				return true;
			}
		}
		return false;
	}

}
