package trains.ui;

import java.awt.BorderLayout;
import java.awt.Font;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.time.Duration;
import java.time.LocalDate;
import java.time.LocalTime;
import java.util.List;
import java.util.Optional;

import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextArea;

import trains.model.*;

public class TrainFrame extends JFrame implements ActionListener {

	private static final long serialVersionUID = 1L;

	private Controller controller;

	private JComboBox<String> departureCityComboBox;
	private JComboBox<String> arrivalCityComboBox;
	private TrainDateSpinner departureDateSpinner;
	private JTextArea trainListArea;

	private JButton searchButton;

	private List<Train> trainsBetween;

	public TrainFrame(Controller controller) {
		super("Ferrovie di Dentinia");
		this.controller = controller;
		initGUI();
		bindData();
	}

	private void bindData() {
		for (String station : controller.getSortedStations()) {
			departureCityComboBox.addItem(station);
			arrivalCityComboBox.addItem(station);
		}
	}

	private void initGUI() {
		setDefaultCloseOperation(EXIT_ON_CLOSE);
		setLayout(new BorderLayout(5, 5));
		setSize(640, 300);

		JPanel leftPanel = new JPanel();
		{
			leftPanel.setLayout(new BorderLayout());
			JPanel upperLeftPanel = new JPanel();
			{
				upperLeftPanel.setLayout(new BoxLayout(upperLeftPanel,
						BoxLayout.PAGE_AXIS));

				JLabel label = new JLabel("Stazione di partenza:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				departureCityComboBox = new JComboBox<>();
				upperLeftPanel.add(departureCityComboBox);

				label = new JLabel("Stazione di arrivo:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				arrivalCityComboBox = new JComboBox<>();
				upperLeftPanel.add(arrivalCityComboBox);

				label = new JLabel("Data del viaggio:");
				label.setAlignmentX(CENTER_ALIGNMENT);
				upperLeftPanel.add(label);
				departureDateSpinner = new TrainDateSpinner();

				upperLeftPanel.add(departureDateSpinner);

				searchButton = new JButton("Cerca");
				searchButton.setAlignmentX(CENTER_ALIGNMENT);
				searchButton.addActionListener(this);
				upperLeftPanel.add(searchButton);
			}
			leftPanel.add(upperLeftPanel, BorderLayout.PAGE_START);
		}
		getContentPane().add(leftPanel, BorderLayout.LINE_START);

		JPanel rightPanel = new JPanel();
		{
			rightPanel.setLayout(new BoxLayout(rightPanel, BoxLayout.PAGE_AXIS));
			rightPanel.add(new JLabel("Soluzioni di viaggio"), BorderLayout.CENTER);
			trainListArea = new JTextArea();
			// trainListArea.setAlignmentX(CENTER_ALIGNMENT);
			trainListArea.setFont(new Font("Courier New", Font.BOLD, 12));
			rightPanel.add(trainListArea, BorderLayout.CENTER);
		}
		getContentPane().add(rightPanel, BorderLayout.CENTER);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		String departureCity = (String) departureCityComboBox.getSelectedItem();
		String arrivalCity = (String) arrivalCityComboBox.getSelectedItem();
		LocalDate date = departureDateSpinner.getDateValue();
		trainListArea.setText("");
		trainsBetween = controller.trainServing(departureCity, arrivalCity, date);
		for (Train t : trainsBetween) {
			Optional<LocalTime> departureTime = Stop
					.searchByFullStationName(t.getStops(), departureCity).get()
					.getDepartureTime();
			Optional<LocalTime> arrivalTime = Stop
					.searchByFullStationName(t.getStops(), arrivalCity).get()
					.getArrivalTime();
			if (departureTime.isPresent() && arrivalTime.isPresent()
					&& departureTime.get().compareTo(arrivalTime.get()) < 0) {
				trainListArea.append(t.getOperator() + " " + t.getNumber() + System.lineSeparator());
				trainListArea.append(String.format("%-8s%-30s%-15s%n", "Da:", departureCity, departureTime.get()));
				trainListArea.append(String.format("%-8s%-30s%-15s%n", "A:",  arrivalCity, arrivalTime.get()));
				trainListArea.append("durata: " + Duration.between(departureTime.get(), arrivalTime.get()).toMinutes()
						              + " minuti" + System.lineSeparator());
				trainListArea.append("--------------------------------------------" + System.lineSeparator());
			}
		}
		trainListArea.append("Non ci sono altre soluzioni" + System.lineSeparator());

	}

}
